@@ -8,7 +8,7 @@ use Mojo::Util 'steady_time';
 use Sys::Hostname 'hostname';
 use Time::HiRes 'usleep';
 
-our $VERSION = '4.004';
+our $VERSION = 'v5.0.0';
 
 has dequeue_interval => 0.5;
 has 'sqlite';
@@ -103,11 +103,18 @@ sub list_jobs {
   my ($self, $offset, $limit, $options) = @_;
 
   my (@where, @where_params);
+  if (defined(my $before = $options->{before})) {
+    push @where, 'id < ?';
+    push @where_params, $before;
+  }
   if (defined(my $ids = $options->{ids})) {
     my $ids_in = join ',', ('?')x@$ids;
     push @where, @$ids ? "id in ($ids_in)" : 'id is null';
     push @where_params, @$ids;
   }
+  if (defined(my $notes = $options->{notes})) {
+    croak 'Listing jobs by existence of notes is unimplemented';
+  }
   if (defined(my $queues = $options->{queues})) {
     my $queues_in = join ',', ('?')x@$queues;
     push @where, @$queues ? "queue in ($queues_in)" : 'queue is null';
@@ -177,6 +184,10 @@ sub list_workers {
   my ($self, $offset, $limit, $options) = @_;
 
   my (@where, @where_params);
+  if (defined(my $before = $options->{before})) {
+    push @where, 'w.id < ?';
+    push @where_params, $before;
+  }
   if (defined(my $ids = $options->{ids})) {
     my $ids_in = join ',', ('?')x@$ids;
     push @where, @$ids ? "w.id in ($ids_in)" : 'w.id is null';
@@ -219,17 +230,26 @@ sub lock {
 
 sub note {
   my ($self, $id, $merge) = @_;
-  my (@set, @set_params);
+  my (@set, @set_params, @remove, @remove_params);
   foreach my $key (keys %$merge) {
     croak qq{Invalid note key '$key'; must not contain '.', '[', or ']'}
       if $key =~ m/[\[\].]/;
-    push @set, q{'$.' || ?}, 'json(?)';
-    push @set_params, $key, {json => $merge->{$key}};
+    if (defined $merge->{$key}) {
+      push @set, q{'$.' || ?}, 'json(?)';
+      push @set_params, $key, {json => $merge->{$key}};
+    } else {
+      push @remove, q{'$.' || ?};
+      push @remove_params, $key;
+    }
   }
   my $json_set = join ', ', @set;
+  my $json_remove = join ', ', @remove;
+  my $set_to = 'notes';
+  $set_to = "json_set($set_to, $json_set)" if @set;
+  $set_to = "json_remove($set_to, $json_remove)" if @remove;
   return !!$self->sqlite->db->query(
-    qq{update minion_jobs set notes = json_set(notes, $json_set)
-       where id = ?}, @set_params, $id
+    qq{update minion_jobs set notes = $set_to where id = ?},
+    @set_params, @remove_params, $id
   )->rows;
 }
 
@@ -293,14 +313,19 @@ sub repair {
 }
 
 sub reset {
-  my $db = shift->sqlite->db;
-  my $tx = $db->begin;
-  $db->query('delete from minion_jobs');
-  $db->query('delete from minion_locks');
-  $db->query('delete from minion_workers');
-  $db->query(q{delete from sqlite_sequence
-    where name in ('minion_jobs','minion_locks','minion_workers')});
-  $tx->commit;
+  my ($self, $options) = (shift, shift // {});
+  my $db = $self->sqlite->db;
+  if ($options->{all}) {
+    my $tx = $db->begin;
+    $db->query('delete from minion_jobs');
+    $db->query('delete from minion_locks');
+    $db->query('delete from minion_workers');
+    $db->query(q{delete from sqlite_sequence
+      where name in ('minion_jobs','minion_locks','minion_workers')});
+    $tx->commit;
+  } elsif ($options->{locks}) {
+    $db->query('delete from minion_locks');
+  }
 }
 
 sub retry_job {
@@ -623,8 +648,7 @@ Transition from C<active> to C<finished> state with or without a result.
 
   my $history = $backend->history;
 
-Get history information for job queue. Note that this method is EXPERIMENTAL and
-might change without warning!
+Get history information for job queue.
 
 These fields are currently available:
 
@@ -660,6 +684,12 @@ These options are currently available:
 
 =over 2
 
+=item before
+
+  before => 23
+
+List only jobs before this id.
+
 =item ids
 
   ids => ['23', '24']
@@ -868,6 +898,12 @@ These options are currently available:
 
 =over 2
 
+=item before
+
+  before => 23
+
+List only workers before this id.
+
 =item ids
 
   ids => ['23', '24']
@@ -950,9 +986,9 @@ defaults to C<1>.
 
   my $bool = $backend->note($job_id, {mojo => 'rocks', minion => 'too'});
 
-Change one or more metadata fields for a job. It is currently an error to
-attempt to set a metadata field with a name containing the characters C<.>,
-C<[>, or C<]>.
+Change one or more metadata fields for a job. Setting a value to C<undef> will
+remove the field. It is currently an error to attempt to set a metadata field
+with a name containing the characters C<.>, C<[>, or C<]>.
 
 =head2 receive
 
@@ -995,10 +1031,28 @@ Repair worker registry and job queue if necessary.
 
 =head2 reset
 
-  $backend->reset;
+  $backend->reset({all => 1});
 
 Reset job queue.
 
+These options are currently available:
+
+=over 2
+
+=item all
+
+  all => 1
+
+Reset everything.
+
+=item locks
+
+  locks => 1
+
+Reset only locks.
+
+=back
+
 =head2 retry_job
 
   my $bool = $backend->retry_job($job_id, $retries);
@@ -1076,15 +1130,13 @@ Number of workers that are currently processing a job.
   delayed_jobs => 100
 
 Number of jobs in C<inactive> state that are scheduled to run at specific time
-in the future. Note that this field is EXPERIMENTAL and might change without
-warning!
+in the future.
 
 =item enqueued_jobs
 
   enqueued_jobs => 100000
 
-Rough estimate of how many jobs have ever been enqueued. Note that this field is
-EXPERIMENTAL and might change without warning!
+Rough estimate of how many jobs have ever been enqueued.
 
 =item failed_jobs
 
